<?php
namespace CADSUP\Controllers;

use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;
use CADSUP\Services\RegiaoService;
use Zeedhi\Framework\DependencyInjection\InstanceManager;
use Zeedhi\Framework\Util\QuerySelector;

class RegiaoController extends \Teknisa\GER\Controllers\TeknisaCrud
{

    #protected $dataSourceName = "regiaor";
    #protected $dataSourceNameToSave = 'regiaosave';
    protected $erroInsert = array();
    protected $connection;

    public function __construct($crudService, ParameterBag $parameterBag, $connection, RegiaoService $service)
    {
        $this->crudService = $crudService;
        $this->parameterBag = $parameterBag;
        $this->con = $connection;
        $this->service = $service;
    }

    public function deleteRegiao(DTO\Request\Row $request, DTO\Response $response)
    {
        $row = $request->getRow();
        $this
            ->service
            ->deleteFilial($row);
        $this
            ->service
            ->deleteMunicipio($row);
        $this
            ->service
            ->deleteRegiao($row);
    }

    public function insertMunicipio(DTO\Request\Row $request, DTO\Response $response)
    {
        $row = $request->getRow();
        $this
            ->service
            ->insertMunicipio($row);
    }

    public function verificaFilial(DTO\Request\Row $request, DTO\Response $response)
    {
        $row = $request->getRow();

        $resposta = $this
            ->service
            ->verificaFilial($row);
        $response->addDataset(new DataSet('resp', array(
            $resposta
        )));
    }

    public function cadastraUnidadeNaRegiao(DTO\Request\Row $request, DTO\Response $response)
    {
        try
        {
            $row = $request->getRow();

            $this
                ->service
                ->cadastraUnidadeNaRegiao($row);

        }
        catch(\Exception $e)
        {
            $response->setError(new Error($e->getMessage() , 500));
        }
    }

    public function deleteFornecedor(DTO\Request $request, DTO\Response $response)
    {
        $connection = InstanceManager::getInstance()->getService('entityManager')->getConnection();
        $queryorcl = "DELETE REFOREG WHERE(INSTR(:CDFORNECED, CDFORNECED)<>0 OR :TFORN = 'T') AND (INSTR(:CDREGIAO, CDREGIAO)<>0 OR :TREG = 'T')";
        $querypost = "DELETE FROM REFOREG WHERE (POSITION(:CDFORNECED IN CDFORNECED) <> 0 OR :TFORN = 'T') AND (POSITION(:CDREGIAO IN CDREGIAO) <> 0 OR :TREG = 'T');";
        $arrayQueries = array (
            'default'   => $queryorcl,
            'pdo_pgsql' => $querypost
            );
        $query = QuerySelector::getQueryByDriver($connection, $arrayQueries);
        $stmt = $this
            ->con
            ->prepare($query);
        $stmt->execute(array(
            "CDFORNECED" => $request->getRow() ['FORNECED'],
            "TFORN" => $request->getRow() ['T_NMFANTFORN'],
            "CDREGIAO" => $request->getRow() ['REGIAO'],
            "TREG" => $request->getRow() ['T_NMREGIAO']
        ));
    }

    public function insertFornecedor(DTO\Request $request, DTO\Response $response)
    {
        $connection = InstanceManager::getInstance()->getService('entityManager')->getConnection();
        $queryorcl = "SELECT FILIAL.CDFILIAL||' - '|| FILIAL.NMFILIAL AS FILIAL, FILIAL.CDREGIAO||' - '|| REGIAO.NMREGIAO AS REGIAO, FORNECEDOR.NRINSJURFORN FORNECEDOR, FORNECEDOR.NMFANTFORN, FORNECEDOR.IDTPIJURFORN FROM FORNECEDOR, REGIAO, FILIAL WHERE (REGIAO.CDREGIAO = FILIAL.CDREGIAO) AND (INSTR(:CDREGIAO, FILIAL.CDREGIAO)<>0 OR :TREG = 'T') AND (INSTR(:CDFORNECED, FORNECEDOR.CDFORNECED)<>0 OR :TFORN = 'T') AND ((EXISTS (SELECT CDFORNECED FROM REFORFIL WHERE REFORFIL.CDFORNECED = FORNECEDOR.CDFORNECED AND REFORFIL.CDFILIAL = FILIAL.CDFILIAL AND REFORFIL.CDPERIENTR IS NOT NULL)) OR (NOT EXISTS (SELECT CDFORNECED FROM REFORFIL WHERE REFORFIL.CDFORNECED = FORNECEDOR.CDFORNECED AND REFORFIL.CDFILIAL = FILIAL.CDFILIAL))) ORDER BY FORNECEDOR.NRINSJURFORN, FILIAL.CDREGIAO, FILIAL.CDFILIAL";
        $querypost = "SELECT FILIAL.CDFILIAL || ' - ' || FILIAL.NMFILIAL AS FILIAL, FILIAL.CDREGIAO || ' - ' || REGIAO.NMREGIAO AS REGIAO, FORNECEDOR.NRINSJURFORN AS FORNECEDOR, FORNECEDOR.NMFANTFORN, FORNECEDOR.IDTPIJURFORN FROM FORNECEDOR JOIN FILIAL ON REGIAO.CDREGIAO = FILIAL.CDREGIAO JOIN REGIAO ON REGIAO.CDREGIAO = FILIAL.CDREGIAO WHERE (POSITION(:CDREGIAO IN FILIAL.CDREGIAO)<>0 OR :TREG = 'T') AND (POSITION(:CDFORNECED IN FORNECEDOR.CDFORNECED)<>0 OR :TFORN = 'T') AND ( EXISTS ( SELECT CDFORNECED FROM REFORFIL WHERE REFORFIL.CDFORNECED = FORNECEDOR.CDFORNECED AND REFORFIL.CDFILIAL = FILIAL.CDFILIAL AND REFORFIL.CDPERIENTR IS NOT NULL ) OR NOT EXISTS ( SELECT CDFORNECED FROM REFORFIL WHERE REFORFIL.CDFORNECED = FORNECEDOR.CDFORNECED AND REFORFIL.CDFILIAL = FILIAL.CDFILIAL ) ) ORDER BY FORNECEDOR.NRINSJURFORN, FILIAL.CDREGIAO, FILIAL.CDFILIAL;";
        $arrayQueries = array (
            'default'   => $queryorcl,
            'pdo_pgsql' => $querypost
        );
        $query = QuerySelector::getQueryByDriver($connection, $arrayQueries);
        $stmt = $this
            ->con
            ->prepare($query);
        $stmt->execute(array(
            "CDFORNECED" => $request->getRow() ['FORNECED'],
            "TFORN" => $request->getRow() ['T_NMFANTFORN'],
            "CDREGIAO" => $request->getRow() ['REGIAO'],
            "TREG" => $request->getRow() ['T_NMREGIAO']
        ));
        $this->erroInsert = $stmt->fetchAll();

        if (empty($this->erroInsert))
        {
            $queryorcl = "INSERT INTO REFOREG(CDFORNECED, CDREGIAO) SELECT FORNECEDOR.CDFORNECED, REGIAO.CDREGIAO FROM FORNECEDOR, REGIAO WHERE (INSTR(:CDFORNECED, FORNECEDOR.CDFORNECED)<>0 OR :TFORN = 'T') AND (INSTR(:CDREGIAO, REGIAO.CDREGIAO)<>0 OR :TREG = 'T') AND NOT EXISTS (SELECT CDREGIAO FROM REFOREG WHERE REFOREG.CDREGIAO = REGIAO.CDREGIAO AND REFOREG.CDFORNECED = FORNECEDOR.CDFORNECED)";
            $querypost = "INSERT INTO REFOREG (CDFORNECED, CDREGIAO) SELECT FORNECEDOR.CDFORNECED, REGIAO.CDREGIAO FROM FORNECEDOR CROSS JOIN REGIAO WHERE (POSITION(:CDFORNECED IN FORNECEDOR.CDFORNECED) <> 0 OR :TFORN = 'T') AND (POSITION(:CDREGIAO IN REGIAO.CDREGIAO) <> 0 OR :TREG = 'T') AND NOT EXISTS ( SELECT 1 FROM REFOREG WHERE REFOREG.CDREGIAO = REGIAO.CDREGIAO AND REFOREG.CDFORNECED = FORNECEDOR.CDFORNECED );";
            $arrayQueries = array (
            'default'   => $queryorcl,
            'pdo_pgsql' => $querypost
            );
            $query = QuerySelector::getQueryByDriver($connection, $arrayQueries);
            $stmtInsert = $this
                ->con
                ->prepare($query);
            $stmtInsert->execute(array(
                "CDFORNECED" => $request->getRow() ['FORNECED'],
                "TFORN" => $request->getRow() ['T_NMFANTFORN'],
                "CDREGIAO" => $request->getRow() ['REGIAO'],
                "TREG" => $request->getRow() ['T_NMREGIAO']
            ));
        }
        else
        {
            $response->addDataset(new DataSet('columnsToReturn', array(
                $this->erroInsert
            )));
        }
    }

    /*
    Fora de uso
    Aguardando para posterior retirada
    */
    /*public function save(DTO\Request $request, DTO\Response $response)
    {	
    	if($request->getDataSet()->getRows()[0]['__is_new']) {
    		$rows = $this->crudService->getRowsToSave($request, $this->dataSourceNameToSave);
    		$this->crudService->save($this->dataSourceNameToSave, $rows);
    	} else {
    		$query = "SELECT count(*) FROM municipio m, regiao r WHERE m.CDREGIAO = r.CDREGIAO AND r.CDREGIAO = :CDREGIAO";
    $stmt = $this->con->prepare($query);
    $stmt->execute(array(
    "CDREGIAO" => $request->getDataSet()->getRows()[0]['CDREGIAO']
    ));
    
    $municipioCount = $stmt->fetchAll()[0]['COUNT(*)'];
    
    $query = "SELECT SGESTADO, CDPAIS FROM regiao WHERE cdregiao = :CDREGIAO";
    $stmt = $this->con->prepare($query);
    $stmt->execute(array(
    "CDREGIAO" => $request->getDataSet()->getRows()[0]['CDREGIAO']
    ));
    
    $oldRegiaoData = $stmt->fetchAll()[0];
    
    if(($oldRegiaoData['SGESTADO'] !== $request->getDataSet()->getRows()[0]['SGESTADO'] ||
    $oldRegiaoData['CDPAIS'] !== $request->getDataSet()->getRows()[0]['CDPAIS']) && 
    $municipioCount > 0){
    $response->setError(new Error("Todos os municipios devem ser apagados antes de trocar o estado ou país", 500));
    } else {
    try {
    	$dataSourceName = $this->dataSourceNameToSave ? $this->dataSourceNameToSave : $this->dataSourceName;
    	$rows = $this->crudService->getRowsToSave($request, $dataSourceName);
    	$this->crudService->save($dataSourceName, $rows);
    } catch (\Exception $e) {
    	$response->setError(new Error($e->getMessage(), 500));
    }
    }
    	}
    }*/

    public function deleteUnidade(DTO\Request\Row $request, DTO\Response $response)
    {
        $rowObj = $request->getRow();
        $rowArray = (array)$rowObj;
        $this
            ->service
            ->deleteUnidade($rowArray);
    }
}

