<?php

namespace CADSUP\Services;

use Doctrine\ORM\EntityManager;
use Teknisa\GER\Modules\Fiscal\Factory;
use Financeiro\Entities\Ccorforn;
use Geral\Entities\Prazopgtforn;
use Setforms2\Shared\Registry;
use \Geral\Util\DateUtil;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO;
use Teknisa\Libs\Util\Utilities;
use Teknisa\Libs\Util\NewCode;

class Sup00800FornecedorService {
    
    protected $environment;
    protected $connection;
    protected $entityManager;

    public function __construct($entityManager, $environment, $fileServerLink, NewCode $newCode) {
        $this->connection = $entityManager->getConnection();
        $this->environment = $environment;
        $this->entityManager = $entityManager;
        $this->fileServerLink = $fileServerLink;
        $this->newCode = $newCode;
    }

    public function deleteFormaPagamento ($row) {
        $nrorg = $this->environment->getNrOrgTrab();
        try {
            $prazosPagamento = $this->entityManager->getRepository('\Teknisa\Model\Geral\Entities\Prazopgtforn')
            ->findBy(array(
                'cdforneced' => $row['CDFORNECED'],
                'nrorg' => $nrorg
            ));
            
            foreach($prazosPagamento as $prazo) {
                $this->entityManager->remove($prazo);
                $this->entityManager->flush();
            }
        }
        catch (\Exception $e) {
            throw $e;
        }
    }

    public function saveContaCorrente ($rows) {
        $nrorg = $this->environment->getNrOrgTrab();
        try {
            $alterado = $rows[0];
            $parans = $rows[1];
            
            $contaCorrenteFornecedor = $this->entityManager->getRepository('\Teknisa\Model\Financeiro\Entities\Ccorforn')
            ->findOneBy(array(
                'cdforneced'     => $parans['CDFORNECED'],
                'nrorg'          => $nrorg,
                'cdbancforn'     => $parans['CDBANCFORN'],
                'cdagenforn'     => $parans['CDAGENFORN'],
                'cdccorforn'     => $parans['CDCCORFORN']
            ));
            $contaCorrenteFornecedor->setCdbancforn($alterado['CDBANCFORN']);
            $contaCorrenteFornecedor->setCdagenforn($alterado['CDAGENFORN']);
            $contaCorrenteFornecedor->setCdccorforn($alterado['CDCCORFORN']);
            $contaCorrenteFornecedor->setNrinscnabfor($alterado['NRINSCNABFOR']);
            $contaCorrenteFornecedor->setIdccorfornpref($alterado['IDCCORFORNPREF']);
            $contaCorrenteFornecedor->setNrorgultatu($nrorg);
            $contaCorrenteFornecedor->setDtultatu(DateUtil::getDataAtual());
             
            $this->entityManager->persist($contaCorrenteFornecedor);
            $this->entityManager->flush();
        }
        catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function retornaDados($row){
        $param = array();
        $param['P_CDFORNECED'] = $row['CDFORNECED'];
        
        $query = "SELECT CDFORNECED FROM OBSERVFORN WHERE CDFORNECED = :P_CDFORNECED";
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
            $retorno = $std->fetchAll();
            return $retorno;
        } catch (Exception $e){
            throw $e;
            return false;
        }
    }
    
    public function saveObservacao ($row) 
    {
		if(empty(self::retornaDados($row))){
            $query = "INSERT INTO OBSERVFORN (DSOBSFORNWEB, CDFORNECED) VALUES (:P_DSOBSFORNWEB, :P_CDFORNECED)";
        }else{
            $query = "UPDATE OBSERVFORN SET DSOBSFORNWEB = :P_DSOBSFORNWEB WHERE CDFORNECED = :P_CDFORNECED";
        }
        
        try {
            $std = $this->connection->prepare($query);
            $std->execute(
                array(
                    'P_DSOBSFORNWEB' => $row['DSOBSFORNWEB'],
                    'P_CDFORNECED'   => $row['CDFORNECED']
                )
            );
        } catch (Exception $e){
            throw $e;
        }             
    }
    
    public function insertPrazo ($row)
    {
		$param = array();
	
        $param['P_CDFORNECED']   = $row['CDFORNECED'];
        $param['P_NRDIASPGTOFO'] = $row['NRDIASPGTOFO'];
        $param['P_VRPERCPGTOFO'] = $row['VRPERCPGTOFO'];
        $param['P_IDATIVO']      = 'S';
        $param['P_NRORG']        = $this->environment->getNrOrg();
        $param['P_CDOPERADOR']   = $this->environment->getCdoperador();
        
        $query = "INSERT INTO PRAZOPGTFORN 
                    (CDFORNECED, 
                    NRDIASPGTOFO, 
                    VRPERCPGTOFO, 
                    DTULTATUPFOR,
                    NRORG,
                    DTINCLUSAO,
                    DTULTATU,
                    NRORGINCLUSAO,
                    CDOPERINCLUSAO,
                    NRORGULTATU,
                    CDOPERULTATU,
                    IDATIVO) 
                  VALUES 
                    (:P_CDFORNECED, 
                    :P_NRDIASPGTOFO,
                    :P_VRPERCPGTOFO,
                    (SYSDATE),
                    :P_NRORG,
                    (SYSDATE),
                    (SYSDATE),
                    :P_NRORG,
                    :P_CDOPERADOR,
                    :P_NRORG,
                    :P_CDOPERADOR,
                    :P_IDATIVO)";
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
        } catch (Exception $e){
            throw $e;
        }  
    }
    
    public function updatePrazo ($row)
    {
		$param = array();
	
        $param['P_CDFORNECED']     = $row['CDFORNECED'];
        $param['P_NRDIASPGTOFO']   = $row['NRDIASPGTOFO'];
        $param['P_VRPERCPGTOFO']   = $row['VRPERCPGTOFO'];
        $param['OLD_NRDIASPGTOFO'] = isset($row['changed_NRDIASPGTOFO']) ? $row['changed_NRDIASPGTOFO'] : $row['NRDIASPGTOFO'];
        $param['OLD_VRPERCPGTOFO'] = isset($row['changed_VRPERCPGTOFO']) ? $row['changed_VRPERCPGTOFO'] : $row['VRPERCPGTOFO'];
        $param['P_CDOPERADOR']     = $this->environment->getCdoperador();
        
        $query = "UPDATE PRAZOPGTFORN SET
                    NRDIASPGTOFO = :P_NRDIASPGTOFO, 
                    VRPERCPGTOFO = :P_VRPERCPGTOFO,
                    DTULTATU = (SYSDATE),
                    CDOPERULTATU = :P_CDOPERADOR
                  WHERE
                    CDFORNECED = :P_CDFORNECED
                    AND NRDIASPGTOFO = :OLD_NRDIASPGTOFO
                    AND VRPERCPGTOFO = :OLD_VRPERCPGTOFO";
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
        } catch (Exception $e){
            throw $e;
        }  
    }
    
    public function getCompacArq($row){
        $param = array();
        $param['P_CDFORNECED'] = $row['CDFORNECED'];
        
        $query = "SELECT IDCOMPARQXMLFOR FROM FORNECEDOR WHERE CDFORNECED = :P_CDFORNECED";
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
            $retorno = $std->fetchAll();
            return $retorno;
        } catch (Exception $e){
            throw $e;
            return false;
        }
    }
    
    public function retornaDadosCompacArq($row){
        $param = array();
        $param['P_CDFORNECED'] = $row['CDFORNECED'];
        
        $query = "SELECT IDCOMPARQXMLFOR FROM FORNECEDOR WHERE CDFORNECED = :P_CDFORNECED";
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
            $retorno = $std->fetchAll();
            return $retorno;
        } catch (Exception $e){
            throw $e;
            return false;
        }
    }
    
    public function salvarCompacArq ($row) {
		$param = array();
		$param['P_IDCOMPARQXMLFOR'] = $row['IDCOMPARQXMLFOR'];
        $param['P_CDFORNECED'] = $row['CDFORNECED'];
        
        
        if(empty(self::retornaDadosCompacArq($row))){
            $query = "INSERT INTO FORNECEDOR (IDCOMPARQXMLFOR) VALUES (:P_IDCOMPARQXMLFOR) WHERE CDFORNECED = :P_CDFORNECED";
            try {
                $std = $this->connection->prepare($query);
                $std->execute($param);
            } catch (Exception $e){
                throw $e;
            }  
        }else{
            $query = "UPDATE FORNECEDOR SET IDCOMPARQXMLFOR = :P_IDCOMPARQXMLFOR WHERE CDFORNECED = :P_CDFORNECED";
            try {
                $std = $this->connection->prepare($query);
                $std->execute($param);
            } catch (Exception $e){
                throw $e;
            } 
        }
    }
    
    public function salvarDigitoCST ($row) {
		$param = array();
        $param['P_CDFORNECED'] = $row['CDFORNECED'];
        $query = "UPDATE FORNECEDOR SET CDCSTDIG1ENT = null WHERE CDFORNECED = :P_CDFORNECED";
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
        } catch (Exception $e){
            throw $e;
        } 
    }
    
    public function salvarContaPagar ($row) {
      
		$param = array();
        $param['P_CDFORNECED'] = $row['CDFORNECED'];
        $param['P_CDTPBAIXAPAG'] = $row['CDTPBAIXAPAG'];
        $param['P_IDAGRUBOLFOR'] = $row['IDAGRUBOLFOR'];
        $param['P_IDTPTITULOCRED'] = $row['IDTPTITULOCRED'];
        $param['P_DSTPTITULOCRED'] = $row['DSTPTITULOCRED'];
        $query = "UPDATE FORNECEDOR SET CDTPBAIXAPAG = :P_CDTPBAIXAPAG, IDAGRUBOLFOR = :P_IDAGRUBOLFOR, IDTPTITULOCRED = :P_IDTPTITULOCRED, DSTPTITULOCRED = :P_DSTPTITULOCRED 
                    WHERE CDFORNECED = :P_CDFORNECED";
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
        } catch (Exception $e){
            throw $e;
        } 
    }
    
    public function getFranqueadora() {
  
        $query = "SELECT CDFILIAL FROM FILIAL WHERE IDFUNCAOFILI = 'F'";
        
        $std = $this->connection->prepare($query);
        try {
            $std->execute();
            $existe = $std->fetch();
            if(isset($existe["CDFILIAL"])) {
             return $existe["CDFILIAL"];   
            }
            else return false;
        }
        catch(\Exception $e) {
            return false;    
        }

    }
    
    public function insertFiliForn($cdFilial, $cdFornecedor) {
        $query = "INSERT
                    INTO FILIFORN
                      (
                        CDFILIAL,
                        CDFORNECED
                      )
                      VALUES
                      (
                        :P_CDFILIAL,
                        :P_CDFORNECED
                      )";
                      
        $params = array(
            "P_CDFILIAL"    => $cdFilial,
            "P_CDFORNECED"  => $cdFornecedor
        );
        
        $std = $this->connection->prepare($query);
        return $std->execute($params);
    }
    
    public function getFiliForn($cdFilial, $cdFornecedor) {
        $query = "SELECT CDFORNECED
                    FROM FILIFORN
                    WHERE CDFILIAL = :P_CDFILIAL
                    AND CDFORNECED = :P_CDFORNECED" ;
                      
        $params = array(
            "P_CDFILIAL"    => $cdFilial,
            "P_CDFORNECED"  => $cdFornecedor
        );
        
        $std = $this->connection->prepare($query);
        $std->execute($params);
        $existe = $std->fetch();
            
        return $existe;
    }
    
    public function getAllFiliForn($cdFornecedor) { 
        $query = "SELECT CDFORNECED
                    FROM FILIFORN
                    WHERE CDFORNECED = :P_CDFORNECED" ;
                      
        $params = array(
            "P_CDFORNECED"  => $cdFornecedor
        );
        
        $std = $this->connection->prepare($query);
        $std->execute($params);
        $existe = $std->fetchAll();
            
        return $existe;
    }
    
    public function exportPayMeth($row) 
    {
        try {
            foreach($row["CDFORNECEDDEST"] as $cdForneced) {
                $this->updatePaymentMethod($row, $cdForneced);
            }
        } catch (Exception $error) {
            throw $error;
        }
    }
    
    private function executeQuery($query, $params) {
        try {
            $std = $this->connection->prepare($query);
            $std->execute($params);
        } catch (Exception $error) {
            throw $error;
        }  
    }
    
    private function updatePaymentMethod($row, $cdForneced)
    {
        if($row["IDTPFORPGTFORIG"] === null) {
            $this->deleteSupplier($cdForneced);
            
            $query = "UPDATE FORNECEDOR 
                         SET IDTPFORPGTFO = NULL,
                             NRPARCPGTOFO = NULL,
                             NRDIAPGFIXFO = NULL
                       WHERE CDFORNECED = :P_CDFORNECED";
        
            $params = [
                "P_CDFORNECED" => $cdForneced
            ];
            
            $this->executeQuery($query, $params);
        } else if($row["IDTPFORPGTFORIG"] === "F") {
            $this->deleteSupplier($cdForneced);  
            
            $query = "UPDATE FORNECEDOR
                       SET IDTPFORPGTFO = 'F',
                           NRPARCPGTOFO = :P_NRPARCPGTOFO,
                           NRDIAPGFIXFO = :P_NRDIAPGFIXFO
                    WHERE  CDFORNECED = :P_CDFORNECED";
        
            $params = [
                "P_NRPARCPGTOFO" => $row["NRPARCPGTOFORIG"],
                "P_NRDIAPGFIXFO" => $row["NRDIAPGFIXFORIG"],
                "P_CDFORNECED" => $cdForneced
            ];                      
            
            $this->executeQuery($query, $params);
        } else {
            $this->deleteSupplier($cdForneced);
            
            $query = "UPDATE FORNECEDOR
                         SET IDTPFORPGTFO = 'V',
                             NRPARCPGTOFO = NULL,
                             NRDIAPGFIXFO = NULL,
                             CDFORMPGTO   = :P_CDFORMPGTO 
                       WHERE CDFORNECED   = :P_CDFORNECED";
            
            $params = [
                "P_CDFORMPGTO" => $row["CDFORMPGTORIG"],
                "P_CDFORNECED" => $cdForneced
            ];
            
            $this->executeQuery($query, $params);
                          
            $deadlines = $this->getDeadlines($row["CDFORNECEDORIG"]);
            
            foreach($deadlines as $deadline) {
                $query = "INSERT INTO PRAZOPGTFORN (CDFORNECED, NRDIASPGTOFO, VRPERCPGTOFO)
                                            VALUES (:CDFORNECED, :NRDIASPGTOFO, :VRPERCPGTOFO)";
                $params = [
                    "CDFORNECED" => $cdForneced,
                    "NRDIASPGTOFO" => $deadline["NRDIASPGTOFO"],
                    "VRPERCPGTOFO" => $deadline["VRPERCPGTOFO"]
                ];
                
                $this->executeQuery($query, $params);
            };
        }
    }
    
    private function getDeadlines($cdForneced)
    {
        $query = "SELECT CDFORNECED, NRDIASPGTOFO, VRPERCPGTOFO
                    FROM PRAZOPGTFORN
                   WHERE CDFORNECED = :P_CDFORNECED";
        $params = [
            "P_CDFORNECED" => $cdForneced
        ];
        
        try {
            $std = $this->connection->prepare($query);
            $std->execute($params);
            return $std->fetchAll();
        } catch (Exception $error) {
            throw $error;
        }
    }
    
    private function deleteSupplier($cdForneced)
    {
        $query = "DELETE FROM PRAZOPGTFORN
                   WHERE CDFORNECED = :P_CDFORNECED";
                   
        $params = [
            "P_CDFORNECED" => $cdForneced
        ];   
        
        try {
            $std = $this->connection->prepare($query);
            $std->execute($params);
        } catch (Exception $error) {
            throw $error;
        }
    }
    
    public function existeFornecedor($cdFornecedor) {
        $query = "SELECT CDFORNECED
                FROM FORNECEDOR
                WHERE CDFORNECED = :P_CDFORNECED" ;
                      
        $params = array(
            "P_CDFORNECED"  => $cdFornecedor
        );
        
        $std = $this->connection->prepare($query);
        $std->execute($params);
        $existe = $std->fetch();
            
        return $existe;
    }
    
    public function deleteFiliForn($cdFilial, $cdFornecedor){
        $query = "DELETE FILIFORN 
                    WHERE CDFILIAL = :P_CDFILIAL
                    AND CDFORNECED = :P_CDFORNECED";
        
        $params = array(
            "P_CDFILIAL"    => $cdFilial,
            "P_CDFORNECED"  => $cdFornecedor
        );
        
        $std = $this->connection->prepare($query);
        return $std->execute($params);
    }
    
    public function updateTpForn($tipo, $cdFornecedor) {
        $query = "UPDATE FORNECEDOR SET IDTPFORN = :P_IDTPFORN WHERE CDFORNECED = :P_CDFORNECED";
        
        $params = array(
            "P_IDTPFORN"    => $tipo,
            "P_CDFORNECED"  => $cdFornecedor
        );
        
        $std = $this->connection->prepare($query);
        return $std->execute($params); 
    }
    
    public function iniciaTransacao(){
        $this->entityManager->beginTransaction();
    }

    public function commit(){
        $this->entityManager->commit();
    }

    public function rollback(){
        $this->entityManager->rollback();
    }
    
    public function getParamForne($cdForneced) {
        $param = array();
        $param['P_CDFORNECED'] = $cdForneced;
        $param['P_NRORG']      = $this->environment->getNrOrg();

        $query = "SELECT CDFORNECED, NRDIASBLOQNFFOR, IDBLQNFRELAFOAF, IDOBRIMPXMLNF,
                         IDOBRCHAVACESFO, IDVERINSFORCHNFE, IDSIMPLESNACIONAL, IMPVENCXML, NRORG
                    FROM PARAMFORNE
                   WHERE CDFORNECED = :P_CDFORNECED
                     AND NRORG = :P_NRORG";
        
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
            return $std->fetchAll();
        } catch (Exception $e) {
            throw $e;
        }
    }
    
    public function getCertificadoDigial($filial)
    {
        $param = array();
        $param['P_CDFILIAL']   = $filial;
        $param['P_NRORG']      = $this->environment->getNrOrg();

        $query =   "SELECT I.DSDIRCERTIFICADO, I.CDSENHACERTIF, F.NMRAZSOCFILI, F.NRINSJURFILI 
                    FROM INSCRESTAD I, FILIAL F
                    WHERE I.CDINSCESTA = F.CDINSCESTA 
                    AND I.CDEMPRESA = F.CDEMPRESA 
                    AND F.CDFILIAL = :P_CDFILIAL
                    AND I.NRORG = :P_NRORG
                    AND DSDIRCERTIFICADO IS NOT NULL";
        
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
            return $std->fetchAll();
        } catch (Exception $e) {
            throw $e;
        }
    }
    
    private function curlCall($url, $payload) {
	    $ch = curl_init(); 
	    curl_setopt($ch, CURLOPT_URL, $url); 
	    curl_setopt($ch, CURLOPT_HEADER, TRUE); 
	    curl_setopt($ch, CURLOPT_NOBODY, TRUE); // remove body 
	    curl_setopt($ch, CURLOPT_RETURNTRANSFER, TRUE); 
	    curl_setopt($ch, CURLOPT_POST, 1); 
	    curl_setopt($ch, CURLOPT_POSTFIELDS, $payload); 
	    curl_setopt($ch, CURLOPT_HTTPHEADER, array(
	        'Content-Type: application/json',
	        'Content-Length: ' . strlen($payload))
	    );
	    $response = curl_exec($ch); 
	    $httpCode = curl_getinfo($ch, CURLINFO_HTTP_CODE); 
	    $header_size = curl_getinfo($ch, CURLINFO_HEADER_SIZE);
	    $header = substr($response, 0, $header_size);
	    $body = substr($response, $header_size);
	    curl_close($ch);
	    
	    return $body;
	}
    
    public function decrypt($text, $salt = 'teknisa') {
          return Utilities::zeedhiDecrypt($text, $salt);
    }
    
    public function registerAddress($rowDadosCadastrais, $uf) {
        try {
            $NRORG = $this->environment->getNrOrgTrab();
            $rowEndereco = $rowDadosCadastrais->ender;

            $params =  array();
            $params["DSLOGRADOURO"] = explode(' ', $rowEndereco->xLgr)[0];

            $retornoSelect = $this->findLogradouroByName($params);

            if ($retornoSelect && count($retornoSelect) > 0) {
                $rowDadosCadastrais->CDLOGRADOURO = $retornoSelect["CDLOGRADOURO"];
                $rowDadosCadastrais->DSLOGRADOURO = $retornoSelect["DSLOGRADOURO"];
            }

            $rowDadosCadastrais->DSENDERECO = (!isset($rowDadosCadastrais->DSLOGRADOURO) || $rowEndereco->xLgr == '')? $rowEndereco->xLgr : substr($rowEndereco->xLgr, strlen($rowDadosCadastrais->DSLOGRADOURO) + 1);

            $params =  array();
            $params["SGESTADO"] = $uf;

            $retornoSelect = $this->findEstadoByCode($params);

            if (count($retornoSelect) > 0) {
                $rowDadosCadastrais->CDPAIS   = '0055';
                $rowDadosCadastrais->NMPAIS   = 'Brasil';
                $rowDadosCadastrais->SGESTADO = $retornoSelect["SGESTADO"];
                $rowDadosCadastrais->NMESTADO = $retornoSelect["NMESTADO"];

                $acentos = array(
                    'á'=>'a', 'à'=>'a', 'ã'=>'a', 'â'=>'a', 'ä'=>'a', 'é'=>'e', 'ê'=>'e', 'è'=>'e', 'ë'=>'e', 'í'=>'i', 'ì'=>'i', 'ï'=>'i',
                    'î'=>'i', 'ó'=>'o', 'ô'=>'o', 'ò'=>'o', 'õ'=>'o', 'ö'=>'o', 'ú'=>'u', 'ü'=>'u', 'ù'=>'u', 'û'=>'u', 'ç'=>'c', 'Á'=>'A',
                    'À'=>'A', 'Ã'=>'A', 'Â'=>'A', 'Ä'=>'A', 'É'=>'E', 'Ê'=>'E', 'È'=>'E', 'Ë'=>'E', 'Í'=>'I', 'Ì'=>'I', 'Ï'=>'I', 'Î'=>'I',
                    'Ó'=>'O', 'Ô'=>'O', 'Õ'=>'O', 'Ò'=>'O', 'Ö'=>'O', 'Ú'=>'U', 'Ù'=>'U', 'Û'=>'U', 'Ü'=>'U', 'Ç'=>'C'
                );

                $params =  array();
                $params["SGESTADO"] = $rowDadosCadastrais->SGESTADO;
                $params["NMMUNICIPIO"] = $rowEndereco->xMun;
                $params["NMMUNICIPIO_SEMACENTO"] = strtr($rowEndereco->xMun, $acentos);

                $retornoSelect = $this->findMunicipioByName($params);

                if (count($retornoSelect) > 0) {
                    $rowDadosCadastrais->NMMUNICIPIO = $retornoSelect["NMMUNICIPIO"];
                    $rowDadosCadastrais->CDMUNICIPIO = $retornoSelect["CDMUNICIPIO"];
                    
                    $params =  array();
                    $params["SGESTADO"] = $rowDadosCadastrais->SGESTADO;
                    $params["CDMUNICIPIO"] = $rowDadosCadastrais->CDMUNICIPIO;
                    $params["NMBAIRRO"] = $rowEndereco->xBairro;
                    $params["NMBAIRRO_SEMACENTO"] = strtr($rowEndereco->xBairro, $acentos);
                    $params['NRORG'] = $NRORG;
                    
                    $retornoSelect = $this->findBairroByName($params);
                    if ($retornoSelect && count($retornoSelect) > 0) {
                        $rowDadosCadastrais->CDBAIRRO = $retornoSelect["CDBAIRRO"];
                        $rowDadosCadastrais->NMBAIRRO = $retornoSelect["NMBAIRRO"];
                    } else if($rowEndereco->xBairro != "") {
                        $trueUpper = array(
                            'á'=>'A', 'à'=>'A', 'ã'=>'A', 'â'=>'A', 'ä'=>'A', 'é'=>'E', 'ê'=>'E', 'è'=>'E', 'ë'=>'Ë', 'í'=>'I', 'ì'=>'I', 'ï'=>'I',
                            'î'=>'I', 'ó'=>'O', 'ô'=>'O', 'ò'=>'O', 'õ'=>'O', 'ö'=>'O', 'ú'=>'U', 'ü'=>'U', 'ù'=>'U', 'û'=>'U', 'ç'=>'C'
                        );

                        $rowEndereco->xBairro = strtoupper(strtr($rowEndereco->xBairro, $trueUpper));
                        $rowDadosCadastrais->CDBAIRRO = $this->insertBairro($rowEndereco->xBairro, $rowDadosCadastrais->CDMUNICIPIO, $rowDadosCadastrais->SGESTADO, '0055');
                        $rowDadosCadastrais->NMBAIRRO = $rowEndereco->xBairro;
                    }
                }
                else {
                    throw new \Exception("O município ".$params["NMMUNICIPIO"]." não foi carregado automaticamente porque não está cadastrado no sistema.");
                }
            }
            else {
                throw new \Exception("O estado ".$params["SGESTADO"]." não foi carregado automaticamente porque não está cadastrado no sistema.");
            }

            return $rowDadosCadastrais;
            
        }
        catch (\Exception $e) {
            throw $e;
        }
    }
    
    private function findEstadoByCode($params) {
        $sql = "SELECT SGESTADO, NMESTADO
                  FROM ESTADO
                 WHERE CDPAIS = '0055'
                   AND UPPER(SGESTADO) = UPPER(:SGESTADO)";

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        return $dataSet = $std->fetch();
    }

    private function findMunicipioByName($params) {
        $sql = "SELECT NMMUNICIPIO, CDMUNICIPIO
                  FROM MUNICIPIO
                 WHERE CDPAIS = '0055'
                   AND SGESTADO = :SGESTADO
                   AND (UPPER(NMMUNICIPIO) = UPPER(:NMMUNICIPIO)
                    OR UPPER(NMMUNICIPIO) = UPPER(:NMMUNICIPIO_SEMACENTO))";

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        return $dataSet = $std->fetch();
    }

    private function findBairroByName($params) {
        $sql = "SELECT NMBAIRRO, CDBAIRRO
                  FROM BAIRRO
                 WHERE CDPAIS = '0055'
                   AND SGESTADO = :SGESTADO
                   AND CDMUNICIPIO = :CDMUNICIPIO
                   AND (UPPER(NMBAIRRO) = UPPER(:NMBAIRRO)
                    OR UPPER(NMBAIRRO) = UPPER(:NMBAIRRO_SEMACENTO))
                   AND NRORG = :NRORG";

        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $dataSet = $std->fetch();
    }

    private function findLogradouroByName($params) {
        $sql = "SELECT CDLOGRADOURO, DSLOGRADOURO
                  FROM LOGRADOURO
                 WHERE UPPER(DSLOGRADOURO) LIKE UPPER(:DSLOGRADOURO) || '%'
                   AND NRORG = :NRORG";

        $params['NRORG'] = $this->environment->getNrOrgTrab();

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        return $dataSet = $std->fetch();
    }
    
    private function insertBairro($NMBAIRRO, $CDMUNICIPIO, $SGESTADO, $CDPAIS) {
        $sql = "INSERT INTO BAIRRO (
                CDBAIRRO,       NMBAIRRO,           CDPAIS,
                SGESTADO,       CDMUNICIPIO,        NRORG,
                DTINCLUSAO,     CDOPERINCLUSAO,     NRORGINCLUSAO)
                VALUES (
                :CDBAIRRO,      :NMBAIRRO,          :CDPAIS,
                :SGESTADO,      :CDMUNICIPIO,       :NRORG,
                SYSDATE,        :CDOPERINCLUSAO,    :NRORG)";

        $CONTADOR = "BAIRRO".$CDPAIS.$SGESTADO.$CDMUNICIPIO;
        $CODIGO = $this->newCode->getCodeByTableName($CONTADOR, $this->environment->getNrOrgTrab());
        $params = array(
            "CDBAIRRO"       => $CODIGO,
            "NMBAIRRO"       => $NMBAIRRO,
            "CDPAIS"         => $CDPAIS,
            "SGESTADO"       => $SGESTADO,
            "CDMUNICIPIO"    => $CDMUNICIPIO,
            "NRORG"          => $this->environment->getNrOrgTrab(),
            "CDOPERINCLUSAO" => $this->environment->getCdOperador()
        );
        $std = $this->connection->prepare($sql);
        $std->execute($params);

        return $CODIGO;
    }

     /**
     * Inativa os fornecedores de acordo com a quantidade de meses recebida desde a data atual menos a quantidade de meses
     *
     * @param int $QUANTIDADE_DE_MESES
     * @return string $alterados
     * 
     */

    public function inativarFornecedores($QUANTIDADE_DE_MESES)
    {
        $params = array(
            "P_DTCORTE" => date("d/m/Y", strtotime('-' . $QUANTIDADE_DE_MESES . ' months'))
        );
        
        $sql = "SELECT FORNECEDOR.CDFORNECED, FORNECEDOR.NRINSJURFORN, FORNECEDOR.NMRAZSOCFORN, FORNECEDOR.NMFANTFORN,  MAX(DTEMISSAO) ULTEMISSAO
                FROM NOTAFISCAL, FORNECEDOR
                WHERE NOTAFISCAL.IDENTRSAIDOP = 'E'
                AND NOTAFISCAL.IDORIGDESTNF = 'F'
                AND ((NOTAFISCAL.IDSISTEMA IN ('EST','DFE') OR IDSISTEMA IS NULL))
                AND (NOTAFISCAL.IDSTATUS NOT IN ('I', 'L', 'D'))
                AND NOTAFISCAL.CDFORNECED = FORNECEDOR.CDFORNECED
                AND FORNECEDOR.IDSITUCADA = 'A'
                GROUP BY FORNECEDOR.CDFORNECED, FORNECEDOR.NRINSJURFORN, FORNECEDOR.NMRAZSOCFORN, FORNECEDOR.NMFANTFORN
                HAVING max(DTEMISSAO) <= :P_DTCORTE
                ORDER BY FORNECEDOR.NRINSJURFORN";

        $std = $this->connection->prepare($sql);

        $std->execute($params);
        $fornecedores = $std->fetchAll();
        
        $alterados = "";

        foreach($fornecedores as $fornecedor) {
            
            $sql = "UPDATE FORNECEDOR
              SET IDSITUCADA = 'I'
              WHERE CDFORNECED = :P_CDFORNECED";

            $params = array(
                "P_CDFORNECED" => $fornecedor['CDFORNECED']
            );

            $std = $this->connection->prepare($sql);

            $std->execute($params);

            $alterados = $this->mask($fornecedor['NRINSJURFORN']) . " | " . $fornecedor["NMRAZSOCFORN"] . " | " . $fornecedor["NMFANTFORN"] . "\n" . $alterados; 
        }

        return $alterados;
    }
    
     /**
     * Coloca a mascara de CNPJ ou CPF de acordo com o tamanho da string
     *
     * @param String $cnpjCpf
     * @return string $maskared - string com mascara
     */

    private function mask($cnpjCpf) {

        $mask = "##.###.###/####-##";
        if(strlen($cnpjCpf) === 11) {
            $mask = '###.###.###-##';
        }

        $maskared = '';
        $k = 0;
        for($i = 0; $i<=strlen($mask)-1; $i++) {
            if($mask[$i] == '#') {
                if(isset($cnpjCpf[$k])) $maskared .= $cnpjCpf[$k++];
            } else {
                if(isset($mask[$i])) $maskared .= $mask[$i];
            }
        }
        return $maskared;
    }
    
}
